
#include "engine.h"
#include "gui.h"
#include "player.h"
#include "netGame.h"
#include "config.h"
#include "engineClient.h"
#include "ai.h"

#include <allegro.h>
#include <stdlib.h>

// EngineClient Constructor
EngineClient::EngineClient(bool human)
{
	ngc.step = 0;
	gui.turn = 0;

	syncFactor = 0;
	syncing = true;
	stage = RECEIVE_INFO;

	type = human ? HUMAN : ARTIFICIAL;
}

bool EngineClient::Connect(int port, char* host)
{
	return (ngc.joinGame(port, host));
}

//! Main Client Function
void EngineClient::oneTurn()
{
	if (!inGame) return;

	if (syncing)
	{
		syncTurn();
	}
	else
	{
		// Usual turn
		gui.handleEvents();

		if (!ngc.SendOrders(gui.orderStack))
		{
			printf("Could not send information to server. Sorry.\n");
			inGame = false;
		}
		ngc.ReceiveOrders(&pa);

		// For every player in the game
		for (int p = 0; p < pa.size; p++)
		{
			// Execute orders in the step
			if (pa.getPlayer(p)->inGame && (!ngc.hasOrders(p)))
			{
				printf("* Package Lost Client Side *\n");
				printf("Player %d did not send orders\n", pa.getPlayer(p)->id);
				printf("step: %d\n", ngc.step);
				inGame = false;
				break;
			}

			while (ngc.hasOrders(p))
			{
				Order* currentOrder = ngc.Pop(p);
				// Player does not need to be in game (Since there's a PLAYER_JOINED order)
				executeOrder(currentOrder);
				delete currentOrder;
			}
		}
	
		runPhysics();
		runLogic();
		gui.Draw(&pa);
	
		ngc.nextStep();
		gui.nextTurn();
	}
}

bool EngineClient::syncedUp()
{
	return (((ngc.step + DELAY) % CYCLE_SIZE) == (gui.turn % CYCLE_SIZE));
}

// Sync up turn (Just receive data)
void EngineClient::syncTurn()
{
	bool update = true;
	GameInformation gameInfo;

	while (update)
	{
		switch (stage)
		{
			case RECEIVE_INFO:
				if (ngc.receiveInfo(&gameInfo))
				{
					printf("Took %d turns (%d ms) to receive gameInfo.\n", gui.turn, gui.turn * TURN_TIME);

					pa = gameInfo.pa;
					ngc.step = gameInfo.step;
					gui.playerId = gameInfo.playerId;

					// Make a step correction
					ngc.step = (ngc.step + gui.turn) % CYCLE_SIZE;
					// And also a turn correction
					gui.turn = (ngc.step + DELAY) % CYCLE_SIZE;

					stage = RECEIVE_ORDERS;

					printf("\n");
					printf("** Game Information **\n");
					printf("Step: %d\n", ngc.step);
					printf("Turn: %d\n", gui.turn);
					printf("Number of players: %d\n", pa.totalPlayers);
					for (int i = 0; i < pa.size; i++)
					{
						Player* myPlayer = pa.getPlayer(i);
						if (myPlayer->inGame)
						{
							printf("Player %d: (%d, %d)\n", myPlayer->id, myPlayer->getX(), myPlayer->getY());
						}
					}
					printf("My id: %d\n", gui.playerId);
					printf("** End **\n");
					printf("\n");
				}
				else
				{
					update = false;
				}
				break;

			case RECEIVE_ORDERS:
				ngc.ReceiveOrders(&pa);
				update = false;
				break;
		}
	}
	
	// Notice that netGame does not move to the next step
	gui.nextTurn();
	syncFactor++;

	if (syncFactor == JOIN_DELAY)
	{
		// Execute all the undone orders
		// Until the difference between step and turn is DELAY
		while (!syncedUp())
		{
			// For every player in the game
			for (int p = 0; p < pa.size; p++)
			{
				// Execute orders in the step
				while (ngc.hasOrders(p))
				{
					Order* currentOrder = ngc.Pop(p);
					executeOrder(currentOrder);
					delete currentOrder;
				}
			}

			runPhysics();
			runLogic();
			ngc.nextStep();
		}
		
		// Now, the client is really in the game
		syncing = false;
		pa.getPlayer(gui.playerId)->inGame = true;
		ngc.initializeMyStack(gui.playerId);

		if (type == ARTIFICIAL)
		{
			gui.setAI(&pa);
		}
		printf("Done syncing.\n");
	}
}

// Auxiliary for allegro timer
void RunClient(void* myEngineClient)
{
	EngineClient* that = (EngineClient*) myEngineClient;
	that->oneTurn();
}
END_OF_FUNCTION(RunClient);

void EngineClient::Run()
{
	LOCK_FUNCTION(RunClient);
	install_param_int(RunClient, (void*)this, TURN_TIME);
	while (inGame)
	{
		rest(TURN_TIME);
	}
	remove_param_int(RunClient, (void*)this);
}
