
#include "gui.h"
#include "order.h"
#include "player.h"
#include "config.h"
#include "ai.h"

#include <allegro.h>
#include <stdio.h>
#include <string.h>


//! Gui Functions

Gui::Gui()
{
	loadMap();
	buffer = create_bitmap(XBUFFER, YBUFFER);
	quitting = false;
	showStats = false;
	ai = NULL;
}

Gui::~Gui()
{
	if (buffer) destroy_bitmap(buffer);
	if (map) destroy_bitmap(map);
	if (ai) delete ai;
}

void Gui::InitializeAllegro()
{
	allegro_init();

	install_timer();
	install_keyboard();
	install_mouse();
	
	set_color_depth(16);
	
	if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, XSCREEN, YSCREEN, 0, 0))
	{
		printf("Graphic mode error.\n");
		exit(1);
	}

	if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL))
	{
		printf("Sound error.\n");
		exit (2);
	}
}

void Gui::setAI (PlayerArray* pa)
{
	ai = new AI();
	ai->Init(pa, playerId);
}

void Gui::updateCatcher (Player* p)
{
	ai->updateCatcher(p);
}

void Gui::nextTurn()
{
	turn++;
	if (turn == CYCLE_SIZE) turn = 0;
}

void Gui::handleEvents()
{
	bool didSomething = false;

	if (quitting) return;

	//! Local Events
	if (key[KEY_ESC])
	{
		orderStack.Push(new Order(Order::QUIT, playerId, turn));
		quitting = true;
		return;
	}
	showStats = key[KEY_TAB];

	//! Shared Events
	if (ai)
	{
		// Artificial Intelligence Events
		ai->RunAI(orderStack, turn);
	}
	else
	{
		// Human Events
		if (key[KEY_UP])
		{
			orderStack.Push(new Order(Order::MOVE_FOWARD, playerId, turn));
			didSomething = true;
		}
	
		if (key[KEY_DOWN])
		{
			orderStack.Push(new Order(Order::MOVE_BACKWARD, playerId, turn));
			didSomething = true;
		}
	
		if (key[KEY_LEFT])
		{
			orderStack.Push(new Order(Order::MOVE_LEFT, playerId, turn));
			didSomething = true;
		}
	
		if (key[KEY_RIGHT])
		{
			orderStack.Push(new Order(Order::MOVE_RIGHT, playerId, turn));
			didSomething = true;
		}
		if (!didSomething)
		{
			orderStack.Push(new Order(Order::NULL_ORDER, playerId, turn));
		}
	}
}

void Gui::drawPlayer(Player* myPlayer)
{
	int color;

	if (myPlayer->isRunner())
	{
		color = makecol(0, 200, 0);
	}
	else if (myPlayer->isCatcher())
	{
		color = makecol(200, 0, 0);
	}
	else
	{
		color = makecol(220, 220, 0);
	}

	circle(buffer, myPlayer->getX(), myPlayer->getY(), PLAYER_RADIUS, color);

	if (myPlayer->id == playerId)
	{
		rect(buffer, xScroll(myPlayer)+1, yScroll(myPlayer)+1, xScroll(myPlayer) + XPLAY_AREA -1, yScroll(myPlayer) + YPLAY_AREA -1, color);
		circlefill(buffer, myPlayer->getX(), myPlayer->getY(), (int) (PLAYER_RADIUS * 0.5), color);
	}
}

// Put the map in a buffer that will be reused everytime
void Gui::loadMap()
{
	map = create_bitmap(XBUFFER, YBUFFER);
	srand(777);
	for (int i = 0; i < 777; i++)
	{
		putpixel(map, rand() % XBUFFER, rand() % YBUFFER, makecol(255, 255, 255));
	}
}

// Put the map buffer into the main buffer
void Gui::drawMap()
{
	blit(map, buffer, 0, 0, 0, 0, XBUFFER, YBUFFER);
}

void Gui::drawInformation(PlayerArray* pa)
{
	Player* myPlayer = pa->getPlayer(playerId);
	int x = xScroll(myPlayer);
	int y = yScroll(myPlayer);

	int fcol = makecol(255, 255, 255);
	int bgcol = makecol(0, 0, 0);

	rect(buffer, x+1, y+1, x+640-1, y+480-1, makecol(255, 255, 255));

	//TODO define this shit
	textprintf_ex(buffer, font, x+80, y+80, fcol, bgcol, "ID");
	textprintf_ex(buffer, font, x+130, y+80, fcol, bgcol, "NAME");
	textprintf_ex(buffer, font, x+280, y+80, fcol, bgcol, "BEING CATCHED");
	textprintf_ex(buffer, font, x+430, y+80, fcol, bgcol, "(x, y)");

	for (int i = 0; i < pa->size; i++)
	{
		Player* p = pa->getPlayer(i);
		if (p->inGame)
		{
			textprintf_ex(buffer, font, x+80, y+150+(40 * p->id), fcol, bgcol, "%d", p->id);
			textprintf_ex(buffer, font, x+130, y+150+(40 * p->id), fcol, bgcol, "%s", p->name);
			textprintf_ex(buffer, font, x+280, y+150+(40 * p->id), fcol, bgcol, "%d", p->beingCatched);
			textprintf_ex(buffer, font, x+430, y+150+(40 * p->id), fcol, bgcol, "(%4d, %4d)", p->getX(), p->getY());
		}
	}

	textprintf_ex(buffer, font, x+5, y+465, fcol, bgcol, "GUI TURN: %d", turn);
	textprintf_ex(buffer, font, x+380, y+465, fcol, bgcol, "ID: %d", playerId);
}

int Gui::xScroll(Player* myPlayer)
{
	int x = (myPlayer->getX() - XPLAY_AREA/2);

	if (x < 0)
	{
		return 0;
	}
	if (x > (XBUFFER - XPLAY_AREA))
	{
		return (XBUFFER - XPLAY_AREA);
	}
	return x;
}

int Gui::yScroll(Player* myPlayer)
{
	int y = (myPlayer->getY() - YPLAY_AREA/2);

	if (y < 0)
	{
		return 0;
	}
	if (y > (YBUFFER - YPLAY_AREA))
	{
		return (YBUFFER - YPLAY_AREA);
	}
	return y;
}

void Gui::BlitOnScreen(Player* myPlayer)
{
	/* TODO isso ta blitando o jogo na area que o jogo vai rolar,
	 porem, deve ser blitado antes em um outro buffer que vai
	 ter algumas informacoes ali do ladinho */

	blit(buffer, screen, xScroll(myPlayer), yScroll(myPlayer), 5, 5, XPLAY_AREA, YPLAY_AREA);
	// Destroy the buffer and create a new one for the next blit
	destroy_bitmap(buffer);
	buffer = create_bitmap(XBUFFER, YBUFFER);
}

//! Main draw function
void Gui::Draw(PlayerArray* pa)
{
	drawMap();
	for (int i = 0; i < pa->size; i++)
	{
		if (pa->getPlayer(i)->inGame)
		{
			drawPlayer(pa->getPlayer(i));
		}
	}
	if (showStats) drawInformation(pa);
	BlitOnScreen(pa->getPlayer(playerId));
}
