
#include "netGame.h"
#include "fifoStack.h"
#include "player.h"
#include "order.h"
#include "config.h"

#include <stdlib.h>

//! NetGame Functions

NetGame::NetGame()
{
	step = 0;
	for (int i = 0; i < CYCLE_SIZE; i++)
	{
		for (int j = 0; j < MAX_PLAYERS; j++)
		{
			ordersQueue[i][j].Initialize();
		}
	}
}

void NetGame::Push(Order* myOrder)
{
	ordersQueue[myOrder->turn][myOrder->playerId].Push(myOrder);
}

Order* NetGame::Pop(short playerId)
{
	return ordersQueue[step][playerId].Pop();
}

bool NetGame::hasOrders(short playerId)
{
	return (ordersQueue[step][playerId].isEmpty() == false);
}

void NetGame::nextStep()
{
	step++;
	if (step == CYCLE_SIZE) step = 0;
}

void NetGame::initializePlayerStack(short playerId)
{
	// Initialize DELAY rounds because the player will be syncing up [turn-(turn + JOIN_DELAY)]
	// New player won't be in the game on the [step-turn] period
	for (int i = (step + DELAY); i < (step + DELAY + JOIN_DELAY); i++)
	{
		ordersQueue[i%CYCLE_SIZE][playerId].Push(new Order(Order::NULL_ORDER, playerId, i%CYCLE_SIZE));
	}
}

// Fill the [step-(step+DELAY)] gap with null orders
void NetGame::initializeMyStack(short myId)
{
	for (int i = step; i < (step + DELAY); i++)
	{
		ordersQueue[i%CYCLE_SIZE][myId].Push(new Order(Order::NULL_ORDER, myId, i%CYCLE_SIZE));
	}
}

void NetGame::checkStack()
{
	printf("Checking stack...\n");
	for (int i = DELAY; i < CYCLE_SIZE; i++)
	{
		for (int p = 0; p < MAX_PLAYERS; p++)
		{
			if (ordersQueue[i][p].isEmpty() == false)
			{
				printf("Missing order by player %d  (He did it on turn: %d)\n", p, i);
			}
		}
	}
	printf("Done.\n");
}

//! NetGameClient Functions

bool NetGameClient::joinGame(int port, char* host)
{
	return (myClient.makeClient(port, host));
}

bool NetGameClient::receiveInfo(GameInformation* gameInfo)
{
	int bytesReceived = myClient.Receive(gameInfo, sizeof(GameInformation));
	if (bytesReceived <= 0)
		return false;
	if (bytesReceived != sizeof(GameInformation))
	{
		printf("Did not received the correct number of bytes. Go speak to Tg about it.\n");
		exit(1);
	}
	return true;
}

bool NetGameClient::SendOrders(FifoStack &myStack)
{
	while (!myStack.isEmpty())
	{
		Order* myOrder = myStack.Pop();
		if (myClient.Send(myOrder, sizeof(Order)) <= 0)
		{
			return false;
		}
	}
	return true;
}

bool NetGameClient::receivePlayer(PlayerArray* pa)
{
	Player* myPlayer = new Player();
	if (myClient.Receive(myPlayer, sizeof(Player)) > 0)
	{
		pa->setPlayer(*myPlayer);
		lastReceivedIndex = myPlayer->id;
		delete myPlayer;
		return true;
	}
	return false;
}

void NetGameClient::ReceiveOrders(PlayerArray* pa)
{
	Order* myOrder = new Order();
	while (myClient.Receive(myOrder, sizeof(Order)) > 0)
	{
		if (myOrder->type == Order::PLAYER_JOINED)
		{
			// FIXME: This while part can unsync stuff
			while (receivePlayer(pa) == false);
			pa->totalPlayers++;
			initializePlayerStack(lastReceivedIndex);
			printf("Novo jogador (totalPlayers: %d - paSize: %d)\n", pa->totalPlayers, pa->size);
		}

		Push(myOrder);
		myOrder = new Order();
	}
	delete myOrder;
}

//! NetGameServer Functions

bool NetGameServer::Accept(PlayerArray* pa, short turn)
{
	if (myServer.Accept())
	{
		GameInformation gameInfo;

		// Create a player for the new client
		short createdIndex = pa->createNewPlayer();
		Player* createdPlayer = pa->getPlayer(createdIndex);

		// Players
		gameInfo.pa = *pa;
		// Current Network Step
		gameInfo.step = step;
		// New client's id
		gameInfo.playerId = createdIndex;

		// Send Game Information to the new client
		myServer.Send(&gameInfo, sizeof(GameInformation), myServer.lastConnectionId);

		// Give the player information to all the clients
		Order* myOrder = new Order(Order::PLAYER_JOINED, createdIndex, turn);
		Push(myOrder);
		initializePlayerStack(createdIndex);

		myServer.SendToAllExcept(myOrder, sizeof(Order), myServer.lastConnectionId);
		myServer.SendToAllExcept(createdPlayer, sizeof(Player), myServer.lastConnectionId);

		// Send the orderQueue from step to turn (step plus delay)
		for (int k = step; k < turn; k++)
		{
			for (int p = 0; p < pa->size; p++)
			{
				if (pa->getPlayer(p)->inGame)
				{
					ordersQueue[k][p].startSearch();
		
					while (ordersQueue[k][p].finishedSearch() == false)
					{
						myOrder = ordersQueue[k][p].Search();
						myServer.Send(myOrder, sizeof(Order), myServer.lastConnectionId);
						// Since the server will still use this order, it won't be deleted
					}
				}
			}
		}

		return true;
	}
	return false;
}

void NetGameServer::SendOrders(FifoStack &myStack)
{
	while (!myStack.isEmpty())
	{
		Order* myOrder = myStack.Pop();
		Push(myOrder);
		myServer.SendToAll(myOrder, sizeof(Order));
	}
}

void NetGameServer::ReceiveAndSendOrders()
{
	Order* myOrder = new Order();

	for (int k = 0; k < myServer.size; k++)
	{
		while (myServer.Receive(myOrder, sizeof(Order), k) > 0)
		{
			Push(myOrder);
			myServer.SendToAll(myOrder, sizeof(Order));
			myOrder = new Order();
		}
	}
	delete myOrder;
}
